@@ -472,6 +472,9 @@ merge_spanning_rects_in_region (GList *region)
                   a = compare->data;
                 }
 
+              /* avoid false positive warning in Clang static analyzer */
+              g_assert (a != delete_me->data);
+
               /* Okay, we can free it now */
               g_free (delete_me->data);
               region = g_list_delete_link (region, delete_me);
@@ -577,7 +580,6 @@ meta_rectangle_get_minimal_spanning_set_for_region (
   *temp_rect = *basic_rect;
   ret = g_list_prepend (NULL, temp_rect);
 
-  strut_iter = all_struts;
   for (strut_iter = all_struts; strut_iter; strut_iter = strut_iter->next)
     {
       GList *rect_iter;
@@ -762,15 +764,6 @@ meta_rectangle_expand_to_avoiding_struts (MetaRectangle       *rect,
     } /* end loop over struts */
 } /* end meta_rectangle_expand_to_avoiding_struts */
 
-void
-meta_rectangle_free_list_and_elements (GList *filled_list)
-{
-  g_list_foreach (filled_list,
-                  (void (*)(gpointer,gpointer))&g_free, /* ew, for ugly */
-                  NULL);
-  g_list_free (filled_list);
-}
-
 gboolean
 meta_rectangle_could_fit_in_region (const GList         *spanning_rects,
                                     const MetaRectangle *rect)
@@ -1784,7 +1777,7 @@ meta_rectangle_find_onscreen_edges (const MetaRectangle *basic_rect,
   ret = g_list_sort (ret, meta_rectangle_edge_cmp);
 
   /* Free the fixed struts list */
-  meta_rectangle_free_list_and_elements (fixed_strut_rects);
+  g_list_free_full (fixed_strut_rects, g_free);
 
   return ret;
 }
