@@ -42,7 +42,7 @@
  */
 
 #define _GNU_SOURCE
-#define _SVID_SOURCE /* for putenv() and some signal-related functions */
+#define _XOPEN_SOURCE /* for putenv() and some signal-related functions */
 
 #include <config.h>
 #include "main.h"
@@ -52,6 +52,7 @@
 #include "ui.h"
 #include "session.h"
 #include "prefs.h"
+#include "core.h"
 
 #include <glib-object.h>
 #include <glib/gprintf.h>
@@ -385,7 +386,9 @@ sigterm_handler (int signum)
 }
 
 static gboolean
-on_sigterm (void)
+on_sigterm (GIOChannel   *source,
+            GIOCondition condition,
+            gpointer     user_data)
 {
   meta_quit (META_EXIT_SUCCESS);
   return FALSE;
@@ -437,7 +440,7 @@ main (int argc, char **argv)
 
   channel = g_io_channel_unix_new (sigterm_pipe_fds[0]);
   g_io_channel_set_flags (channel, G_IO_FLAG_NONBLOCK, NULL);
-  g_io_add_watch (channel, G_IO_IN, (GIOFunc) on_sigterm, NULL);
+  g_io_add_watch (channel, G_IO_IN, on_sigterm, NULL);
   g_io_channel_set_close_on_unref (channel, TRUE);
   g_io_channel_unref (channel);
 
@@ -482,9 +485,6 @@ main (int argc, char **argv)
 
   meta_ui_init (&argc, &argv);
 
-  /* must be after UI init so we can override GDK handlers */
-  meta_errors_init ();
-
   /* Load prefs */
   meta_prefs_init ();
   meta_prefs_add_listener (prefs_changed_callback, NULL);
@@ -660,6 +660,9 @@ prefs_changed_callback (MetaPreference pref,
       meta_display_set_cursor_theme (meta_prefs_get_cursor_theme (),
 				     meta_prefs_get_cursor_size ());
       break;
+    case META_PREF_ICON_SIZE:
+      meta_invalidate_all_icons();
+      break;
     default:
       /* handled elsewhere or otherwise */
       break;
