@@ -27,11 +27,11 @@
 #include "window-private.h"
 #include "frame-private.h"
 
-void meta_window_place (MetaWindow *window,
-                        MetaFrameGeometry *fgeom,
-                        int         x,
-                        int         y,
-                        int        *new_x,
-                        int        *new_y);
+void meta_window_place (MetaWindow       *window,
+                        MetaFrameBorders *borders,
+                        int               x,
+                        int               y,
+                        int              *new_x,
+                        int              *new_y);
 
 #endif
