@@ -35,6 +35,7 @@
 #include "prefs.h"
 #include "workspace.h"
 #include "keybindings.h"
+#include "prefs.h"
 #include "stack.h"
 #include "xprops.h"
 #include "compositor.h"
@@ -47,6 +48,7 @@
 #endif
 
 #include <X11/Xatom.h>
+#include <cairo/cairo-xlib.h>
 #include <locale.h>
 #include <string.h>
 #include <stdio.h>
@@ -118,14 +120,15 @@ set_supported_hint (MetaScreen *screen)
 static int
 set_wm_icon_size_hint (MetaScreen *screen)
 {
+  int icon_size = meta_prefs_get_icon_size();
 #define N_VALS 6
   gulong vals[N_VALS];
 
   /* min width, min height, max w, max h, width inc, height inc */
-  vals[0] = META_ICON_WIDTH;
-  vals[1] = META_ICON_HEIGHT;
-  vals[2] = META_ICON_WIDTH;
-  vals[3] = META_ICON_HEIGHT;
+  vals[0] = icon_size; /* width */
+  vals[1] = icon_size; /* height */
+  vals[2] = icon_size; /* width */
+  vals[3] = icon_size; /* height */
   vals[4] = 0;
   vals[5] = 0;
 
@@ -648,9 +651,8 @@ meta_screen_free (MetaScreen *screen,
   meta_screen_ungrab_keys (screen);
 
 #ifdef HAVE_STARTUP_NOTIFICATION
-  g_slist_foreach (screen->startup_sequences,
-                   (GFunc) sn_startup_sequence_unref, NULL);
-  g_slist_free (screen->startup_sequences);
+  g_slist_free_full (screen->startup_sequences,
+                     (GDestroyNotify) sn_startup_sequence_unref);
   screen->startup_sequences = NULL;
 
   if (screen->startup_sequence_timeout != 0)
@@ -749,9 +751,8 @@ list_windows (MetaScreen *screen)
       else
         {
 	  info->xwindow = children[i];
+          result = g_list_prepend (result, info);
 	}
-
-      result = g_list_prepend (result, info);
     }
 
   if (children)
@@ -798,8 +799,7 @@ meta_screen_manage_all_windows (MetaScreen *screen)
     }
   meta_stack_thaw (screen->stack);
 
-  g_list_foreach (windows, (GFunc)g_free, NULL);
-  g_list_free (windows);
+  g_list_free_full (windows, g_free);
 
   meta_display_ungrab (screen->display);
 }
@@ -839,8 +839,7 @@ meta_screen_composite_all_windows (MetaScreen *screen)
 
   meta_stack_thaw (screen->stack);
 
-  g_list_foreach (windows, (GFunc)g_free, NULL);
-  g_list_free (windows);
+  g_list_free_full (windows, g_free);
 #endif
 }
 
@@ -1214,60 +1213,60 @@ meta_screen_update_cursor (MetaScreen *screen)
   XFreeCursor (screen->display->xdisplay, xcursor);
 }
 
-#define MAX_PREVIEW_SCALE 10.0
+#define MAX_PREVIEW_SCREEN_FRACTION 0.33
+#define MAX_PREVIEW_SIZE 300
 
-static GdkPixbuf *
-get_window_pixbuf (MetaWindow *window,
-                   int        *width,
-                   int        *height)
+static cairo_surface_t *
+get_window_surface (MetaWindow *window)
 {
-  MetaDisplay *display;
-  cairo_surface_t *surface;
-  GdkPixbuf *pixbuf, *scaled;
+  cairo_surface_t *surface, *scaled;
+  cairo_t *cr;
   const MetaXineramaScreenInfo *current;
+  int width, height, max_columns, max_size;
   double ratio;
 
-  display = window->display;
-  surface = meta_compositor_get_window_surface (window->display->compositor,
-                                                window);
-  if (surface == None)
+  surface = meta_compositor_get_window_surface (window->display->compositor, window);
+
+  if (surface == NULL)
     return NULL;
 
-  meta_error_trap_push (display);
-
-  pixbuf = meta_ui_get_pixbuf_from_surface (surface);
-  cairo_surface_destroy (surface);
-
-  if (meta_error_trap_pop_with_return (display, FALSE) != Success)
-    g_clear_object (&pixbuf);
-
-  if (pixbuf == NULL)
-    return NULL;
-
-  *width = gdk_pixbuf_get_width (pixbuf);
-  *height = gdk_pixbuf_get_height (pixbuf);
+  width = cairo_xlib_surface_get_width (surface);
+  height = cairo_xlib_surface_get_height (surface);
 
   current = meta_screen_get_current_xinerama (window->screen);
+  max_columns = meta_prefs_get_alt_tab_max_columns ();
 
-  /* Scale pixbuf to max dimension based on monitor size */
-  if (*width > *height)
+  /* Scale surface to fit current screen */
+  if (width > height)
     {
-      int max_preview_width = current->rect.width / MAX_PREVIEW_SCALE;
-      ratio = ((double) *width) / max_preview_width;
-      *width = (int) max_preview_width;
-      *height = (int) (((double) *height) / ratio);
+      max_size = MIN (MAX_PREVIEW_SIZE, current->rect.width / max_columns * MAX_PREVIEW_SCREEN_FRACTION);
+      ratio = ((double) width) / max_size;
+      width = (int) max_size;
+      height = (int) (((double) height) / ratio);
     }
   else
     {
-      int max_preview_height = current->rect.height / MAX_PREVIEW_SCALE;
-      ratio = ((double) *height) / max_preview_height;
-      *height = (int) max_preview_height;
-      *width = (int) (((double) *width) / ratio);
+      max_size = MIN (MAX_PREVIEW_SIZE, current->rect.height / max_columns * MAX_PREVIEW_SCREEN_FRACTION);
+      ratio = ((double) height) / max_size;
+      height = (int) max_size;
+      width = (int) (((double) width) / ratio);
     }
 
-  scaled = gdk_pixbuf_scale_simple (pixbuf, *width, *height,
-                                    GDK_INTERP_BILINEAR);
-  g_object_unref (pixbuf);
+  meta_error_trap_push (window->display);
+  scaled = cairo_surface_create_similar (surface,
+                                         cairo_surface_get_content (surface),
+                                         width, height);
+  if (meta_error_trap_pop_with_return (window->display, FALSE) != Success)
+    return NULL;
+
+  cr = cairo_create (scaled);
+  cairo_scale (cr, 1/ratio, 1/ratio);
+  cairo_set_source_surface (cr, surface, 0, 0);
+  cairo_paint (cr);
+
+  cairo_destroy (cr);
+  cairo_surface_destroy (surface);
+
   return scaled;
 }
 
@@ -1282,6 +1281,7 @@ meta_screen_ensure_tab_popup (MetaScreen      *screen,
   int len;
   int i;
   gint border;
+  int scale;
 
   if (screen->tab_popup)
     return;
@@ -1292,11 +1292,13 @@ meta_screen_ensure_tab_popup (MetaScreen      *screen,
                                         screen->active_workspace);
 
   len = g_list_length (tab_list);
+  scale = gdk_window_get_scale_factor (gdk_get_default_root_window ());
 
   entries = g_new (MetaTabEntry, len + 1);
   entries[len].key = NULL;
   entries[len].title = NULL;
   entries[len].icon = NULL;
+  entries[len].win_surface = NULL;
 
   border = meta_prefs_show_tab_border() ? BORDER_OUTLINE_TAB |
     BORDER_OUTLINE_WINDOW : BORDER_OUTLINE_TAB;
@@ -1306,45 +1308,64 @@ meta_screen_ensure_tab_popup (MetaScreen      *screen,
     {
       MetaWindow *window;
       MetaRectangle r;
-      GdkPixbuf *win_pixbuf = NULL;
-      int width = 0, height = 0;
 
       window = tmp->data;
 
       entries[i].key = (MetaTabEntryKey) window->xwindow;
       entries[i].title = window->title;
+      entries[i].win_surface = NULL;
 
-      /* Only get the pixbuf if the user does NOT have
-         compositing-fast-alt-tab-set to true
-         in GSettings. There is an obvious lag when the pixbuf is
-         retrieved. */
-      if (!meta_prefs_get_compositing_fast_alt_tab())
-        win_pixbuf = get_window_pixbuf (window, &width, &height);
+      entries[i].icon = g_object_ref (window->icon);
 
-      if (win_pixbuf == NULL)
-        entries[i].icon = g_object_ref (window->icon);
-      else
+      /* Only get the window thumbnail surface if the user has a compositor
+       * enabled and does NOT have compositing-fast-alt-tab-set to true in
+       * GSettings. There is an obvious lag when the surface is retrieved. */
+      if (meta_prefs_get_compositing_manager() && !meta_prefs_get_compositing_fast_alt_tab())
         {
-          int icon_width, icon_height, t_width, t_height;
+          cairo_surface_t *win_surface;
+
+          /* Get window thumbnail */
+          win_surface = get_window_surface (window);
+
+          if (win_surface != NULL)
+            {
+              cairo_surface_t *surface, *icon;
+              cairo_t *cr;
+              int width, height, icon_width, icon_height;
+
 #define ICON_OFFSET 6
 
-          icon_width = gdk_pixbuf_get_width (window->icon);
-          icon_height = gdk_pixbuf_get_height (window->icon);
+              width = cairo_xlib_surface_get_width (win_surface);
+              height = cairo_xlib_surface_get_height (win_surface);
 
-          t_width = width + ICON_OFFSET;
-          t_height = height + ICON_OFFSET;
+              /* Create a new surface to overlap the window icon into */
+              surface = cairo_surface_create_similar (win_surface,
+                                                      cairo_surface_get_content (win_surface),
+                                                      width, height);
 
-          entries[i].icon = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-                                            t_width, t_height);
-          gdk_pixbuf_fill (entries[i].icon, 0x00000000);
-          gdk_pixbuf_copy_area (win_pixbuf, 0, 0, width, height,
-                                entries[i].icon, 0, 0);
-          g_object_unref (win_pixbuf);
-          gdk_pixbuf_composite (window->icon, entries[i].icon,
-                                t_width - icon_width, t_height - icon_height,
-                                icon_width, icon_height,
-                                t_width - icon_width, t_height - icon_height,
-                                1.0, 1.0, GDK_INTERP_BILINEAR, 255);
+              cr = cairo_create (surface);
+              cairo_set_source_surface (cr, win_surface, 0, 0);
+              cairo_paint (cr);
+
+              /* Get the window icon as a surface */
+              icon = gdk_cairo_surface_create_from_pixbuf (window->icon, scale, NULL);
+
+              icon_width = cairo_image_surface_get_width (icon) / scale;
+              icon_height = cairo_image_surface_get_height (icon) / scale;
+
+              /* Overlap the window icon surface over the window thumbnail */
+              cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+              cairo_set_source_surface (cr, icon,
+                                        width  - icon_width - ICON_OFFSET,
+                                        height - icon_height - ICON_OFFSET);
+              cairo_paint (cr);
+
+              entries[i].win_surface = surface;
+
+              cairo_destroy (cr);
+              cairo_surface_destroy (icon);
+              cairo_surface_destroy (win_surface);
+            }
         }
 
       entries[i].blank = FALSE;
@@ -1386,11 +1407,16 @@ meta_screen_ensure_tab_popup (MetaScreen      *screen,
 
   screen->tab_popup = meta_ui_tab_popup_new (entries,
                                              len,
-                                             5, /* FIXME */
+                                             meta_prefs_get_alt_tab_max_columns(),
+                                             meta_prefs_get_alt_tab_expand_to_fit_title(),
                                              border);
 
   for (i = 0; i < len; i++)
-    g_object_unref (entries[i].icon);
+    {
+      g_object_unref (entries[i].icon);
+      if (entries[i].win_surface)
+        cairo_surface_destroy (entries[i].win_surface);
+    }
 
   g_free (entries);
 
@@ -1424,6 +1450,7 @@ meta_screen_ensure_workspace_popup (MetaScreen *screen)
   entries[len].key = NULL;
   entries[len].title = NULL;
   entries[len].icon = NULL;
+  entries[len].win_surface = NULL;
 
   i = 0;
   while (i < len)
@@ -1438,6 +1465,7 @@ meta_screen_ensure_workspace_popup (MetaScreen *screen)
           entries[i].key = (MetaTabEntryKey) workspace;
           entries[i].title = meta_workspace_get_name (workspace);
           entries[i].icon = NULL;
+          entries[i].win_surface = NULL;
           entries[i].blank = FALSE;
 
           g_assert (entries[i].title != NULL);
@@ -1447,6 +1475,7 @@ meta_screen_ensure_workspace_popup (MetaScreen *screen)
           entries[i].key = NULL;
           entries[i].title = NULL;
           entries[i].icon = NULL;
+          entries[i].win_surface = NULL;
           entries[i].blank = TRUE;
         }
       entries[i].hidden = FALSE;
@@ -1458,6 +1487,7 @@ meta_screen_ensure_workspace_popup (MetaScreen *screen)
   screen->tab_popup = meta_ui_tab_popup_new (entries,
                                              len,
                                              layout.cols,
+                                             FALSE, /* expand_for_titles */
                                              BORDER_OUTLINE_WORKSPACE);
 
   g_free (entries);
@@ -2168,10 +2198,7 @@ meta_screen_calc_workspace_layout (MetaScreen          *screen,
 
   grid = g_new (int, grid_area);
 
-  current_row = -1;
-  current_col = -1;
   i = 0;
-
   switch (screen->starting_corner)
     {
     case META_SCREEN_TOPLEFT:
