@@ -27,11 +27,12 @@
 #include <stdio.h>
 #include <X11/Xutil.h> /* Just for the definition of the various gravities */
 #include <time.h>      /* To initialize random seed */
+#include <math.h>
 
 #define NUM_RANDOM_RUNS 10000
 
 static void
-init_random_ness ()
+init_random_ness (void)
 {
   srand(time(NULL));
 }
@@ -100,7 +101,7 @@ new_xinerama_edge (int x, int y, int width, int height, int side_type)
 }
 
 static void
-test_area ()
+test_area (void)
 {
   MetaRectangle temp;
   int i;
@@ -117,7 +118,7 @@ test_area ()
 }
 
 static void
-test_intersect ()
+test_intersect (void)
 {
   MetaRectangle a = {100, 200,  50,  40};
   MetaRectangle b = {  0,  50, 110, 152};
@@ -145,7 +146,7 @@ test_intersect ()
 }
 
 static void
-test_equal ()
+test_equal (void)
 {
   MetaRectangle a = {10, 12, 4, 18};
   MetaRectangle b = a;
@@ -164,7 +165,7 @@ test_equal ()
 }
 
 static void
-test_overlap_funcs ()
+test_overlap_funcs (void)
 {
   MetaRectangle temp1, temp2;
   int i;
@@ -187,7 +188,7 @@ test_overlap_funcs ()
 }
 
 static void
-test_basic_fitting ()
+test_basic_fitting (void)
 {
   MetaRectangle temp1, temp2, temp3;
   int i;
@@ -357,7 +358,7 @@ get_xinerama_edges (int which_xinerama_set, int which_strut_set)
   ret = meta_rectangle_find_nonintersected_xinerama_edges (&screenrect, xins, struts);
 
   free_strut_list (struts);
-  meta_rectangle_free_list_and_elements (xins);
+  g_list_free_full (xins, g_free);
 
   return ret;
 }
@@ -586,7 +587,7 @@ verify_lists_are_equal (GList *code, GList *answer)
 }
 
 static void
-test_regions_okay ()
+test_regions_okay (void)
 {
   GList* region;
   GList* tmp;
@@ -598,8 +599,8 @@ test_regions_okay ()
   tmp = NULL;
   tmp = g_list_prepend (tmp, new_meta_rect (0, 0, 1600, 1200));
   verify_lists_are_equal (region, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (region, g_free);
 
   /*************************************************************/
   /* Make sure test region 1 has the right spanning rectangles */
@@ -609,8 +610,8 @@ test_regions_okay ()
   tmp = g_list_prepend (tmp, new_meta_rect (0, 20,  400, 1180));
   tmp = g_list_prepend (tmp, new_meta_rect (0, 20, 1600, 1140));
   verify_lists_are_equal (region, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (region, g_free);
 
   /*************************************************************/
   /* Make sure test region 2 has the right spanning rectangles */
@@ -623,8 +624,8 @@ test_regions_okay ()
   tmp = g_list_prepend (tmp, new_meta_rect (   0,   20,  800, 1130));
   tmp = g_list_prepend (tmp, new_meta_rect (   0,   20, 1600, 1080));
   verify_lists_are_equal (region, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (region, g_free);
 
   /*************************************************************/
   /* Make sure test region 3 has the right spanning rectangles */
@@ -649,8 +650,8 @@ test_regions_okay ()
   printf ("%s vs. %s\n", region_list, tmp_list);
 #endif
   verify_lists_are_equal (region, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (region, g_free);
 
   /*************************************************************/
   /* Make sure test region 4 has the right spanning rectangles */
@@ -659,8 +660,8 @@ test_regions_okay ()
   tmp = NULL;
   tmp = g_list_prepend (tmp, new_meta_rect ( 800,   20,  800, 1180));
   verify_lists_are_equal (region, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (region, g_free);
 
   /*************************************************************/
   /* Make sure test region 5 has the right spanning rectangles */
@@ -678,7 +679,7 @@ test_regions_okay ()
 }
 
 static void
-test_region_fitting ()
+test_region_fitting (void)
 {
   GList* region;
   MetaRectangle rect;
@@ -692,7 +693,7 @@ test_region_fitting ()
       g_assert (meta_rectangle_contained_in_region (region, &rect) == FALSE ||
                 meta_rectangle_could_fit_in_region (region, &rect) == TRUE);
     }
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (region, g_free);
 
   /* Do some manual tests too */
   region = get_screen_region (1);
@@ -709,20 +710,20 @@ test_region_fitting ()
   g_assert (meta_rectangle_could_fit_in_region (region, &rect));
   g_assert (!meta_rectangle_contained_in_region (region, &rect));
 
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (region, g_free);
 
   region = get_screen_region (2);
   rect = meta_rect (1000, 50, 600, 1100);
   g_assert (meta_rectangle_could_fit_in_region (region, &rect));
   g_assert (!meta_rectangle_contained_in_region (region, &rect));
 
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (region, g_free);
 
   printf ("%s passed.\n", G_STRFUNC);
 }
 
 static void
-test_clamping_to_region ()
+test_clamping_to_region (void)
 {
   GList* region;
   MetaRectangle rect;
@@ -746,7 +747,7 @@ test_clamping_to_region ()
       g_assert (meta_rectangle_could_fit_in_region (region, &rect) == TRUE);
       g_assert (rect.x == temp.x && rect.y == temp.y);
     }
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (region, g_free);
 
   /* Do some manual tests too */
   region = get_screen_region (1);
@@ -812,7 +813,7 @@ test_clamping_to_region ()
                                            &min_size);
   g_assert (rect.width == 100 && rect.height == 999999);
 
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (region, g_free);
 
   printf ("%s passed.\n", G_STRFUNC);
 }
@@ -839,7 +840,7 @@ rect_overlaps_region (const GList         *spanning_rects,
 gboolean time_to_print = FALSE;
 
 static void
-test_clipping_to_region ()
+test_clipping_to_region (void)
 {
   GList* region;
   MetaRectangle rect, temp;
@@ -856,7 +857,7 @@ test_clipping_to_region ()
           g_assert (meta_rectangle_contained_in_region (region, &rect) == TRUE);
         }
     }
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (region, g_free);
 
   /* Do some manual tests too */
   region = get_screen_region (2);
@@ -895,13 +896,13 @@ test_clipping_to_region ()
                                  &rect);
   g_assert (meta_rectangle_equal (&rect, &temp));
 
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (region, g_free);
 
   printf ("%s passed.\n", G_STRFUNC);
 }
 
 static void
-test_shoving_into_region ()
+test_shoving_into_region (void)
 {
   GList* region;
   MetaRectangle rect, temp;
@@ -918,7 +919,7 @@ test_shoving_into_region ()
           g_assert (meta_rectangle_contained_in_region (region, &rect));
         }
     }
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (region, g_free);
 
   /* Do some manual tests too */
   region = get_screen_region (2);
@@ -965,7 +966,7 @@ test_shoving_into_region ()
                                     &rect);
   g_assert (meta_rectangle_equal (&rect, &temp));
 
-  meta_rectangle_free_list_and_elements (region);
+  g_list_free_full (region, g_free);
 
   printf ("%s passed.\n", G_STRFUNC);
 }
@@ -1018,7 +1019,7 @@ verify_edge_lists_are_equal (GList *code, GList *answer)
 }
 
 static void
-test_find_onscreen_edges ()
+test_find_onscreen_edges (void)
 {
   GList* edges;
   GList* tmp;
@@ -1038,8 +1039,8 @@ test_find_onscreen_edges ()
   tmp = g_list_prepend (tmp, new_screen_edge (1600,    0, 0, 1200, right));
   tmp = g_list_prepend (tmp, new_screen_edge (   0,    0, 0, 1200, left));
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   /*************************************************/
   /* Make sure test region 1 has the correct edges */
@@ -1053,8 +1054,8 @@ test_find_onscreen_edges ()
   tmp = g_list_prepend (tmp, new_screen_edge ( 400, 1160, 0,   40, right));
   tmp = g_list_prepend (tmp, new_screen_edge (   0,   20, 0, 1180, left));
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   /*************************************************/
   /* Make sure test region 2 has the correct edges */
@@ -1074,8 +1075,8 @@ test_find_onscreen_edges ()
   tmp = g_list_prepend (tmp, new_screen_edge ( 450, 1150, 0,   50, left));
   tmp = g_list_prepend (tmp, new_screen_edge (   0,   20, 0, 1180, left));
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   /*************************************************/
   /* Make sure test region 3 has the correct edges */
@@ -1109,8 +1110,8 @@ test_find_onscreen_edges ()
 #endif
 
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   /*************************************************/
   /* Make sure test region 4 has the correct edges */
@@ -1122,8 +1123,8 @@ test_find_onscreen_edges ()
   tmp = g_list_prepend (tmp, new_screen_edge (1600,   20, 0, 1180, right));
   tmp = g_list_prepend (tmp, new_screen_edge ( 800,   20, 0, 1180, left));
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   /*************************************************/
   /* Make sure test region 5 has the correct edges */
@@ -1131,8 +1132,8 @@ test_find_onscreen_edges ()
   edges = get_screen_edges (5);
   tmp = NULL;
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   /*************************************************/
   /* Make sure test region 6 has the correct edges */
@@ -1144,14 +1145,14 @@ test_find_onscreen_edges ()
   tmp = g_list_prepend (tmp, new_screen_edge (1600,   40, 0,  1160, right));
   tmp = g_list_prepend (tmp, new_screen_edge (   0,   40, 0,  1160, left));
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   printf ("%s passed.\n", G_STRFUNC);
 }
 
 static void
-test_find_nonintersected_xinerama_edges ()
+test_find_nonintersected_xinerama_edges (void)
 {
   GList* edges;
   GList* tmp;
@@ -1167,8 +1168,8 @@ test_find_nonintersected_xinerama_edges ()
   edges = get_xinerama_edges (0, 0);
   tmp = NULL;
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   /*************************************************************************/
   /* Make sure test xinerama set 2 for with region 1 has the correct edges */
@@ -1178,8 +1179,8 @@ test_find_nonintersected_xinerama_edges ()
   tmp = g_list_prepend (tmp, new_xinerama_edge (   0,  600, 1600, 0, bottom));
   tmp = g_list_prepend (tmp, new_xinerama_edge (   0,  600, 1600, 0, top));
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   /*************************************************************************/
   /* Make sure test xinerama set 1 for with region 2 has the correct edges */
@@ -1197,8 +1198,8 @@ test_find_nonintersected_xinerama_edges ()
          big_buffer1, big_buffer2);
 #endif
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   /*************************************************************************/
   /* Make sure test xinerama set 3 for with region 3 has the correct edges */
@@ -1212,8 +1213,8 @@ test_find_nonintersected_xinerama_edges ()
   tmp = g_list_prepend (tmp, new_xinerama_edge ( 800,  675, 0,  425, right));
   tmp = g_list_prepend (tmp, new_xinerama_edge ( 800,  675, 0,  525, left));
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   /*************************************************************************/
   /* Make sure test xinerama set 3 for with region 4 has the correct edges */
@@ -1224,8 +1225,8 @@ test_find_nonintersected_xinerama_edges ()
   tmp = g_list_prepend (tmp, new_xinerama_edge ( 800,  600,  800, 0, top));
   tmp = g_list_prepend (tmp, new_xinerama_edge ( 800,  600,  0, 600, right));
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   /*************************************************************************/
   /* Make sure test xinerama set 3 for with region 5has the correct edges */
@@ -1233,14 +1234,14 @@ test_find_nonintersected_xinerama_edges ()
   edges = get_xinerama_edges (3, 5);
   tmp = NULL;
   verify_edge_lists_are_equal (edges, tmp);
-  meta_rectangle_free_list_and_elements (tmp);
-  meta_rectangle_free_list_and_elements (edges);
+  g_list_free_full (tmp, g_free);
+  g_list_free_full (edges, g_free);
 
   printf ("%s passed.\n", G_STRFUNC);
 }
 
 static void
-test_gravity_resize ()
+test_gravity_resize (void)
 {
   MetaRectangle oldrect, rect, temp;
 
@@ -1340,8 +1341,9 @@ test_gravity_resize ()
   printf ("%s passed.\n", G_STRFUNC);
 }
 
+#define EPSILON 0.000000001
 static void
-test_find_closest_point_to_line ()
+test_find_closest_point_to_line (void)
 {
   double x1, y1, x2, y2, px, py, rx, ry;
   double answer_x, answer_y;
@@ -1354,7 +1356,7 @@ test_find_closest_point_to_line ()
                                                   x2,  y2,
                                                   px,  py,
                                                   &rx, &ry);
-  g_assert (rx == answer_x && ry == answer_y);
+  g_assert (fabs (rx - answer_x) < EPSILON && fabs (ry - answer_y) < EPSILON);
 
   /* Special test for x1 == x2, so that slop of line is infinite */
   x1 =  3.0;  y1 =  49.0;
@@ -1365,7 +1367,7 @@ test_find_closest_point_to_line ()
                                                   x2,  y2,
                                                   px,  py,
                                                   &rx, &ry);
-  g_assert (rx == answer_x && ry == answer_y);
+  g_assert (fabs (rx - answer_x) < EPSILON && fabs (ry - answer_y) < EPSILON);
 
   /* Special test for y1 == y2, so perp line has slope of infinity */
   x1 =  3.14;  y1 =   7.0;
@@ -1376,7 +1378,7 @@ test_find_closest_point_to_line ()
                                                   x2,  y2,
                                                   px,  py,
                                                   &rx, &ry);
-  g_assert (rx == answer_x && ry == answer_y);
+  g_assert (fabs (rx - answer_x) < EPSILON && fabs (ry - answer_y) < EPSILON);
 
   /* Test when we the point we want to be closest to is actually on the line */
   x1 =  3.0;  y1 =  49.0;
@@ -1387,13 +1389,13 @@ test_find_closest_point_to_line ()
                                                   x2,  y2,
                                                   px,  py,
                                                   &rx, &ry);
-  g_assert (rx == answer_x && ry == answer_y);
+  g_assert (fabs (rx - answer_x) < EPSILON && fabs (ry - answer_y) < EPSILON);
 
   printf ("%s passed.\n", G_STRFUNC);
 }
 
 int
-main()
+main(void)
 {
   init_random_ness ();
   test_area ();
