@@ -60,6 +60,9 @@ typedef enum
   META_PREF_MATE_ANIMATIONS,
   META_PREF_CURSOR_THEME,
   META_PREF_CURSOR_SIZE,
+  META_PREF_ICON_SIZE,
+  META_PREF_ALT_TAB_MAX_COLUMNS,
+  META_PREF_ALT_TAB_EXPAND_TO_FIT_TITLE,
   META_PREF_COMPOSITING_MANAGER,
   META_PREF_COMPOSITING_FAST_ALT_TAB,
   META_PREF_RESIZE_WITH_RIGHT_BUTTON,
@@ -67,6 +70,7 @@ typedef enum
   META_PREF_CENTER_NEW_WINDOWS,
   META_PREF_ALLOW_TILING,
   META_PREF_ALLOW_TOP_TILING,
+  META_PREF_ALLOW_TILE_CYCLING,
   META_PREF_FORCE_FULLSCREEN,
   META_PREF_PLACEMENT_MODE,
   META_PREF_SHOW_DESKTOP_SKIP_LIST
@@ -104,6 +108,7 @@ gboolean                    meta_prefs_get_mate_accessibility (void);
 gboolean                    meta_prefs_get_mate_animations    (void);
 gboolean                    meta_prefs_get_allow_tiling       (void);
 gboolean                    meta_prefs_get_allow_top_tiling   (void);
+gboolean                    meta_prefs_get_allow_tile_cycling (void);
 
 const char*                 meta_prefs_get_command            (int i);
 
@@ -129,6 +134,9 @@ void        meta_prefs_change_workspace_name (int         i,
 
 const char* meta_prefs_get_cursor_theme      (void);
 int         meta_prefs_get_cursor_size       (void);
+int         meta_prefs_get_icon_size         (void);
+int         meta_prefs_get_alt_tab_max_columns (void);
+gboolean    meta_prefs_get_alt_tab_expand_to_fit_title (void);
 gboolean    meta_prefs_get_compositing_manager (void);
 gboolean    meta_prefs_get_compositing_fast_alt_tab (void);
 gboolean    meta_prefs_get_center_new_windows  (void);
