@@ -42,7 +42,7 @@
 
 #include <cairo-xlib.h>
 
-G_DEFINE_TYPE (MetaFrames, meta_frames, GTK_TYPE_INVISIBLE);
+G_DEFINE_TYPE (MetaFrames, meta_frames, GTK_TYPE_WINDOW);
 
 #define DEFAULT_INNER_BUTTON_BORDER 3
 
@@ -50,10 +50,7 @@ static void meta_frames_class_init (MetaFramesClass *klass);
 static void meta_frames_init       (MetaFrames      *frames);
 static void meta_frames_destroy    (GtkWidget       *object);
 static void meta_frames_finalize   (GObject         *object);
-static void meta_frames_style_set  (GtkWidget       *widget,
-                                    GtkStyle        *prev_style);
-static void meta_frames_realize    (GtkWidget       *widget);
-static void meta_frames_unrealize  (GtkWidget       *widget);
+static void meta_frames_style_updated  (GtkWidget   *widget);
 
 static void meta_frames_update_prelit_control (MetaFrames      *frames,
                                                MetaUIFrame     *frame,
@@ -131,9 +128,7 @@ meta_frames_class_init (MetaFramesClass *class)
   gobject_class->constructor = meta_frames_constructor;
   gobject_class->finalize = meta_frames_finalize;
   widget_class->destroy = meta_frames_destroy;
-  widget_class->style_set = meta_frames_style_set;
-  widget_class->realize = meta_frames_realize;
-  widget_class->unrealize = meta_frames_unrealize;
+  widget_class->style_updated = meta_frames_style_updated;
 
   widget_class->draw = meta_frames_draw;
   widget_class->destroy_event = meta_frames_destroy_event;
@@ -205,7 +200,7 @@ create_style_context (MetaFrames  *frames,
       provider = gtk_css_provider_get_named (theme_name, variant);
       gtk_style_context_add_provider (style,
                                       GTK_STYLE_PROVIDER (provider),
-                                      GTK_STYLE_PROVIDER_PRIORITY_THEME);
+                                      GTK_STYLE_PROVIDER_PRIORITY_SETTINGS);
     }
 
   if (theme_name)
@@ -234,20 +229,20 @@ static void
 update_style_contexts (MetaFrames *frames)
 {
   GtkStyleContext *style;
-  GList *variants, *variant;
+  GList *variant_list, *variant;
 
   if (frames->normal_style)
     g_object_unref (frames->normal_style);
   frames->normal_style = create_style_context (frames, NULL);
 
-  variants = g_hash_table_get_keys (frames->style_variants);
-  for (variant = variants; variant; variant = variants->next)
+  variant_list = g_hash_table_get_keys (frames->style_variants);
+  for (variant = variant_list; variant; variant = variant->next)
     {
       style = create_style_context (frames, (char *)variant->data);
       g_hash_table_insert (frames->style_variants,
                            g_strdup (variant->data), style);
     }
-  g_list_free (variants);
+  g_list_free (variant_list);
 }
 static void
 meta_frames_init (MetaFrames *frames)
@@ -483,8 +478,7 @@ reattach_style_func (gpointer key, gpointer value, gpointer data)
 }
 
 static void
-meta_frames_style_set  (GtkWidget *widget,
-                        GtkStyle  *prev_style)
+meta_frames_style_updated (GtkWidget *widget)
 {
   MetaFrames *frames;
 
@@ -496,7 +490,7 @@ meta_frames_style_set  (GtkWidget *widget,
   g_hash_table_foreach (frames->frames,
                         reattach_style_func, frames);
 
-  GTK_WIDGET_CLASS (meta_frames_parent_class)->style_set (widget, prev_style);
+  GTK_WIDGET_CLASS (meta_frames_parent_class)->style_updated (widget);
 }
 
 static void
@@ -622,12 +616,25 @@ MetaFrames*
 meta_frames_new (void)
 {
   GdkScreen *screen;
+  MetaFrames *frames;
 
   screen = gdk_display_get_default_screen (gdk_display_get_default ());
 
-  return g_object_new (META_TYPE_FRAMES,
-                       "screen", screen,
-                       NULL);
+  frames = g_object_new (META_TYPE_FRAMES,
+                         "screen", screen,
+                         "type", GTK_WINDOW_POPUP,
+                         NULL);
+
+  /* Put the window at an arbitrary offscreen location; the one place
+   * it can't be is at -100x-100, since the meta_window_new() will
+   * mistake it for a window created via meta_create_offscreen_window()
+   * and ignore it, and we need this window to get frame-synchronization
+   * messages so that GTK+'s style change handling works.
+   */
+  gtk_window_move (GTK_WINDOW (frames), -200, -200);
+  gtk_window_resize (GTK_WINDOW (frames), 1, 1);
+
+  return frames;
 }
 
 /* In order to use a style with a window it has to be attached to that
@@ -736,20 +743,6 @@ meta_frames_unmanage_window (MetaFrames *frames,
     meta_warning ("Frame 0x%lx not managed, can't unmanage\n", xwindow);
 }
 
-static void
-meta_frames_realize (GtkWidget *widget)
-{
-  if (GTK_WIDGET_CLASS (meta_frames_parent_class)->realize)
-    GTK_WIDGET_CLASS (meta_frames_parent_class)->realize (widget);
-}
-
-static void
-meta_frames_unrealize (GtkWidget *widget)
-{
-  if (GTK_WIDGET_CLASS (meta_frames_parent_class)->unrealize)
-    GTK_WIDGET_CLASS (meta_frames_parent_class)->unrealize (widget);
-}
-
 static MetaUIFrame*
 meta_frames_lookup_window (MetaFrames *frames,
                            Window      xwindow)
@@ -1678,7 +1671,7 @@ meta_frames_button_press_event (GtkWidget      *widget,
       (control == META_FRAME_CONTROL_MAXIMIZE ||
        control == META_FRAME_CONTROL_UNMAXIMIZE))
     {
-      MetaGrabOp op = META_GRAB_OP_NONE;
+      MetaGrabOp op;
 
       switch (control)
         {
@@ -1690,8 +1683,6 @@ meta_frames_button_press_event (GtkWidget      *widget,
           op = op > META_GRAB_OP_CLICKING_MAXIMIZE_HORIZONTAL ? META_GRAB_OP_CLICKING_MAXIMIZE : op;
           break;
         case META_FRAME_CONTROL_UNMAXIMIZE:
-          op = META_GRAB_OP_CLICKING_UNMAXIMIZE + event->button - 1;
-          op = op > META_GRAB_OP_CLICKING_UNMAXIMIZE_HORIZONTAL ? META_GRAB_OP_CLICKING_UNMAXIMIZE : op;
           op = META_GRAB_OP_CLICKING_UNMAXIMIZE;
           break;
         case META_FRAME_CONTROL_DELETE:
@@ -1723,7 +1714,7 @@ meta_frames_button_press_event (GtkWidget      *widget,
           break;
         default:
           g_assert_not_reached ();
-          break;
+          op = META_GRAB_OP_NONE;
         }
 
       meta_core_begin_grab_op (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
