@@ -28,8 +28,9 @@
 #include "util.h"
 #include "core.h"
 #include "tabpopup.h"
-/* FIXME these two includes are 100% broken ...
- */
+#include "theme.h"
+#include "prefs.h"
+/* FIXME these two includes are 100% broken... */
 #include "../core/workspace.h"
 #include "../core/frame-private.h"
 #include "draw-workspace.h"
@@ -37,8 +38,8 @@
 #include <gdk/gdkx.h>
 #include <math.h>
 
-#define OUTSIDE_SELECT_RECT 2
-#define INSIDE_SELECT_RECT 2
+#define OUTSIDE_SELECT_RECT 8
+#define INSIDE_SELECT_RECT 8
 
 typedef struct _TabEntry TabEntry;
 
@@ -46,7 +47,10 @@ struct _TabEntry
 {
   MetaTabEntryKey  key;
   char            *title;
+  gint             grid_left;
+  gint             grid_top;
   GdkPixbuf       *icon, *dimmed_icon;
+  cairo_surface_t *win_surface;
   GtkWidget       *widget;
   GdkRectangle     rect;
   GdkRectangle     inner_rect;
@@ -56,6 +60,7 @@ struct _TabEntry
 struct _MetaTabPopup
 {
   GtkWidget *window;
+  GtkWidget *grid;
   GtkWidget *label;
   GList *current;
   GList *entries;
@@ -64,11 +69,12 @@ struct _MetaTabPopup
   gint border;
 };
 
-static GtkWidget* selectable_image_new (GdkPixbuf *pixbuf);
+static GtkWidget* selectable_image_new (GdkPixbuf *pixbuf, cairo_surface_t *win_surface);
 static void       select_image         (GtkWidget *widget);
 static void       unselect_image       (GtkWidget *widget);
 
-static GtkWidget* selectable_workspace_new (MetaWorkspace *workspace);
+static GtkWidget* selectable_workspace_new (MetaWorkspace *workspace,
+                                                           int entry_count);
 static void       select_workspace         (GtkWidget *widget);
 static void       unselect_workspace       (GtkWidget *widget);
 
@@ -77,6 +83,7 @@ outline_window_draw (GtkWidget *widget,
                      cairo_t   *cr,
                      gpointer   data)
 {
+  GdkRGBA black = { 0.0, 0.0, 0.0, 1.0 };
   MetaTabPopup *popup;
   TabEntry *te;
 
@@ -90,6 +97,9 @@ outline_window_draw (GtkWidget *widget,
 
   te = popup->current_selected_entry;
 
+  gdk_cairo_set_source_rgba (cr, &black);
+  cairo_paint (cr);
+
   cairo_set_line_width (cr, 1.0);
   cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
 
@@ -108,6 +118,21 @@ outline_window_draw (GtkWidget *widget,
   return FALSE;
 }
 
+static void
+popup_window_screen_changed (GtkWidget *widget,
+                             GdkScreen *old_screen G_GNUC_UNUSED,
+                             gpointer   data G_GNUC_UNUSED)
+{
+  /* To check if the display supports alpha channels, get the visual */
+  GdkScreen *screen = gtk_widget_get_screen (widget);
+  GdkVisual *visual = gdk_screen_get_rgba_visual (screen);
+
+  if (!visual)
+    visual = gdk_screen_get_system_visual (screen);
+
+  gtk_widget_set_visual(widget, visual);
+}
+
 static GdkPixbuf*
 dimm_icon (GdkPixbuf *pixbuf)
 {
@@ -160,16 +185,10 @@ tab_entry_new (const MetaTabEntry *entry,
     {
       gchar *str;
       gchar *tmp;
-      gchar *formatter = "%s";
 
       str = meta_g_utf8_strndup (entry->title, 4096);
 
-      if (entry->hidden)
-        {
-          formatter = "[%s]";
-        }
-
-      tmp = g_markup_printf_escaped (formatter, str);
+      tmp = g_markup_printf_escaped (entry->hidden ? "[%s]" : "%s", str);
       g_free (str);
       str = tmp;
 
@@ -189,6 +208,7 @@ tab_entry_new (const MetaTabEntry *entry,
     }
   te->widget = NULL;
   te->icon = entry->icon;
+  te->win_surface = entry->win_surface;
   te->blank = entry->blank;
   te->dimmed_icon = NULL;
   if (te->icon)
@@ -217,10 +237,11 @@ MetaTabPopup*
 meta_ui_tab_popup_new (const MetaTabEntry *entries,
                        int                 entry_count,
                        int                 width,
+                       gboolean            expand_for_titles,
                        gint                border)
 {
   MetaTabPopup *popup;
-  int i, left, right, top, bottom;
+  int i, left, top;
   int height;
   GtkWidget *grid;
   GtkWidget *vbox;
@@ -229,39 +250,59 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   int max_label_width; /* the actual max width of the labels we create */
   AtkObject *obj;
   GdkScreen *screen;
-  int screen_width, scale;
+  int scale;
+
+  GtkShadowType frame_shadow = GTK_SHADOW_OUT;
 
   popup = g_new (MetaTabPopup, 1);
 
-  popup->outline_window = gtk_window_new (GTK_WINDOW_POPUP);
-
   screen = gdk_display_get_default_screen (gdk_display_get_default ());
-  gtk_window_set_screen (GTK_WINDOW (popup->outline_window),
-                         screen);
 
-  gtk_widget_set_app_paintable (popup->outline_window, TRUE);
-  gtk_widget_realize (popup->outline_window);
+  if (border & BORDER_OUTLINE_WINDOW)
+    {
+      popup->outline_window = gtk_window_new (GTK_WINDOW_POPUP);
 
-  g_signal_connect (G_OBJECT (popup->outline_window), "draw",
-                    G_CALLBACK (outline_window_draw), popup);
+      gtk_window_set_screen (GTK_WINDOW (popup->outline_window),
+                             screen);
+
+      gtk_widget_set_app_paintable (popup->outline_window, TRUE);
+      gtk_widget_realize (popup->outline_window);
+
+      g_signal_connect (G_OBJECT (popup->outline_window), "draw",
+                        G_CALLBACK (outline_window_draw), popup);
+
+      gtk_widget_show (popup->outline_window);
+    }
+  else
+    popup->outline_window = NULL;
 
   popup->window = gtk_window_new (GTK_WINDOW_POPUP);
 
-  gtk_window_set_screen (GTK_WINDOW (popup->window),
-                         screen);
+  gtk_window_set_screen (GTK_WINDOW (popup->window), screen);
+  gtk_window_set_position (GTK_WINDOW (popup->window), GTK_WIN_POS_CENTER_ALWAYS);
 
-  gtk_window_set_position (GTK_WINDOW (popup->window),
-                           GTK_WIN_POS_CENTER_ALWAYS);
   /* enable resizing, to get never-shrink behavior */
-  gtk_window_set_resizable (GTK_WINDOW (popup->window),
-                            TRUE);
+  gtk_window_set_resizable (GTK_WINDOW (popup->window), TRUE);
+
+  /* This style should only be set for composited mode. */
+  if (meta_prefs_get_compositing_manager ())
+    {
+      frame_shadow = GTK_SHADOW_NONE;
+      gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (popup->window)),
+                                   GTK_STYLE_CLASS_OSD);
+    }
+
+  g_signal_connect (G_OBJECT (popup->window), "screen-changed",
+                    G_CALLBACK (popup_window_screen_changed), NULL);
+
+  popup_window_screen_changed (popup->window, NULL, NULL);
+
   popup->current = NULL;
   popup->entries = NULL;
   popup->current_selected_entry = NULL;
   popup->border = border;
 
   scale = gtk_widget_get_scale_factor (GTK_WIDGET (popup->window));
-  screen_width = WidthOfScreen (gdk_x11_screen_get_xscreen (screen));
   for (i = 0; i < entry_count; ++i)
     {
       TabEntry* new_entry = tab_entry_new (&entries[i], border & BORDER_OUTLINE_WINDOW, scale);
@@ -277,15 +318,14 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
-  grid = gtk_grid_new ();
+  popup->grid = gtk_grid_new ();
+  grid = popup->grid;
 
   frame = gtk_frame_new (NULL);
-  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
-  gtk_container_set_border_width (GTK_CONTAINER (grid), 1);
-  gtk_container_add (GTK_CONTAINER (popup->window),
-                     frame);
-  gtk_container_add (GTK_CONTAINER (frame),
-                     vbox);
+  gtk_frame_set_shadow_type (GTK_FRAME (frame), frame_shadow);
+  gtk_container_set_border_width (GTK_CONTAINER (grid), 16);
+  gtk_container_add (GTK_CONTAINER (popup->window), frame);
+  gtk_container_add (GTK_CONTAINER (frame), vbox);
 
   gtk_box_pack_start (GTK_BOX (vbox), grid, TRUE, TRUE, 0);
 
@@ -298,22 +338,23 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   obj = gtk_widget_get_accessible (popup->label);
   atk_object_set_role (obj, ATK_ROLE_STATUSBAR);
 
-  gtk_widget_set_margin_start (popup->label, 3);
-  gtk_widget_set_margin_end (popup->label, 3);
-  gtk_widget_set_margin_top (popup->label, 3);
-  gtk_widget_set_margin_bottom (popup->label, 3);
+  gtk_widget_set_margin_start (popup->label, 16);
+  gtk_widget_set_margin_end (popup->label, 16);
+  gtk_widget_set_margin_top (popup->label, 0);
+  gtk_widget_set_margin_bottom (popup->label, 16);
 
   gtk_box_pack_end (GTK_BOX (vbox), popup->label, FALSE, FALSE, 0);
 
   max_label_width = 0;
   top = 0;
-  bottom = 1;
+  left = 0;
   tmp = popup->entries;
 
+  gtk_widget_show (popup->label); /* for gtk_widget_get_preferred_size() */
+
   while (tmp && top < height)
     {
       left = 0;
-      right = 1;
 
       while (tmp && left < width)
         {
@@ -333,42 +374,37 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
             {
               if (te->dimmed_icon)
                 {
-                  image = selectable_image_new (te->dimmed_icon);
+                  image = selectable_image_new (te->dimmed_icon, NULL);
                 }
               else
                 {
-                  image = selectable_image_new (te->icon);
+                  image = selectable_image_new (te->icon, te->win_surface);
                 }
 
-              gtk_misc_set_padding (GTK_MISC (image),
-                                    INSIDE_SELECT_RECT + OUTSIDE_SELECT_RECT + 1,
-                                    INSIDE_SELECT_RECT + OUTSIDE_SELECT_RECT + 1);
               gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
               gtk_widget_set_valign (image, GTK_ALIGN_CENTER);
             }
           else
             {
-              image = selectable_workspace_new ((MetaWorkspace *) te->key);
+              image = selectable_workspace_new ((MetaWorkspace *) te->key, entry_count);
             }
 
           te->widget = image;
-
+          te->grid_left = left;
+          te->grid_top  = top;
           gtk_grid_attach (GTK_GRID (grid), te->widget, left, top, 1, 1);
 
           /* Efficiency rules! */
-          gtk_label_set_markup (GTK_LABEL (popup->label),
-                              te->title);
+          gtk_label_set_markup (GTK_LABEL (popup->label), te->title);
           gtk_widget_get_preferred_size (popup->label, &req, NULL);
           max_label_width = MAX (max_label_width, req.width);
 
           tmp = tmp->next;
 
           ++left;
-          ++right;
         }
 
       ++top;
-      ++bottom;
     }
 
   /* remove all the temporary text */
@@ -376,17 +412,48 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   /* Make it so that we ellipsize if the text is too long */
   gtk_label_set_ellipsize (GTK_LABEL (popup->label), PANGO_ELLIPSIZE_END);
 
-  /* Limit the window size to no bigger than screen_width/4 */
-  if (max_label_width>(screen_width/4))
+  int default_window_width = 0; /* 0 == small as possible, truncate label */
+
+  if (expand_for_titles && top <= 1 && left < width)
     {
-      max_label_width = screen_width/4;
+      /* only one row partially filled with columns and additional_columns given
+         => calculate default_window_width to fit max_label_width if possible */
+
+      GtkWidget **dummies = g_try_malloc (sizeof (GtkWidget*) * (width - left));
+      if (dummies)
+        {
+          int j;
+          for (j = 0; j < width - left; ++j)
+            {
+              GtkWidget *dummy = gtk_label_new ("");
+              dummies[j] = dummy;
+              gtk_grid_attach (GTK_GRID (grid), dummy, left + j, top, 1, 1);
+            }
+
+          gtk_grid_set_column_homogeneous (GTK_GRID(grid), TRUE);
+          gtk_widget_set_halign (grid, GTK_ALIGN_CENTER);
+          gtk_widget_show_all (grid); /* for gtk_widget_get_preferred_size */
+
+          GtkRequisition req;
+          gtk_widget_get_preferred_size (grid, &req, NULL);
+          default_window_width = req.width;
+
+          for (j = 0; j < width - left; ++j)
+            {
+              gtk_container_remove (GTK_CONTAINER (grid), dummies[j]);
+            }
+          g_free (dummies);
+
+          /* Limit the window size to no bigger than max_label_width */
+          if (max_label_width < default_window_width)
+            {
+              default_window_width = max_label_width;
+            }
+        }
     }
 
-  max_label_width += 20; /* add random padding */
-
   gtk_window_set_default_size (GTK_WINDOW (popup->window),
-                               max_label_width,
-                               -1);
+                               default_window_width, -1);
 
   return popup;
 }
@@ -446,12 +513,6 @@ static void
 display_entry (MetaTabPopup *popup,
                TabEntry     *te)
 {
-  GdkRectangle rect;
-  cairo_region_t *region;
-  cairo_region_t *inner_region;
-  GdkWindow *window;
-
-
   if (popup->current_selected_entry)
   {
     if (popup->border & BORDER_OUTLINE_TAB)
@@ -469,8 +530,11 @@ display_entry (MetaTabPopup *popup,
 
   if (popup->border & BORDER_OUTLINE_WINDOW)
     {
-      window = gtk_widget_get_window (popup->outline_window);
+      GdkRectangle rect;
+      GdkWindow *window;
+      cairo_region_t *region;
 
+      window = gtk_widget_get_window (popup->outline_window);
       /* Do stuff behind gtk's back */
       gdk_window_hide (window);
       meta_core_increment_event_serial (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
@@ -479,30 +543,17 @@ display_entry (MetaTabPopup *popup,
       rect.x = 0;
       rect.y = 0;
 
-      gdk_window_move_resize (window,
-                              te->rect.x, te->rect.y,
-                              te->rect.width, te->rect.height);
-
-      GdkRGBA black = { 0.0, 0.0, 0.0, 1.0 };
-      gdk_window_set_background_rgba (window, &black);
+      gtk_window_move (GTK_WINDOW (popup->outline_window), te->rect.x, te->rect.y);
+      gtk_window_resize (GTK_WINDOW (popup->outline_window), te->rect.width, te->rect.height);
 
       region = cairo_region_create_rectangle (&rect);
-      inner_region = cairo_region_create_rectangle (&te->inner_rect);
-      cairo_region_subtract (region, inner_region);
-      cairo_region_destroy (inner_region);
+      cairo_region_subtract_rectangle (region, &te->inner_rect);
 
-      gdk_window_shape_combine_region (window,
+      gdk_window_shape_combine_region (gtk_widget_get_window (popup->outline_window),
                                        region,
                                        0, 0);
 
       cairo_region_destroy (region);
-
-      /* This should piss off gtk a bit, but we don't want to raise
-       * above the tab popup.  So, instead of calling gtk_widget_show,
-       * we manually set the window as mapped and then manually map it
-       * with gdk functions.
-       */
-      gtk_widget_set_mapped (popup->outline_window, TRUE);
       gdk_window_show_unraised (window);
     }
 
@@ -548,6 +599,50 @@ meta_ui_tab_popup_backward (MetaTabPopup *popup)
     }
 }
 
+static void
+display_widget (MetaTabPopup *popup,
+                GtkWidget    *w)
+{
+  if (w != NULL) 
+    {
+      GList *c = popup->entries;
+      while (c != NULL && ((TabEntry*)c->data)->widget != w)
+        c = c->next;
+      if (c != NULL) 
+        {
+          popup->current = c;
+          TabEntry *te = c->data;
+          display_entry (popup, te);
+        }
+    }
+}
+
+void
+meta_ui_tab_popup_down (MetaTabPopup *popup)
+{
+  if (popup->current != NULL)
+    {
+      TabEntry *te = popup->current->data;
+      GtkWidget* w = gtk_grid_get_child_at (GTK_GRID(popup->grid), 
+                                            te->grid_left,
+                                            te->grid_top + 1);
+      display_widget (popup, w);
+    }
+}
+
+void
+meta_ui_tab_popup_up (MetaTabPopup *popup)
+{
+  if (popup->current != NULL)
+    {
+      TabEntry *te = popup->current->data;
+      GtkWidget* w = gtk_grid_get_child_at (GTK_GRID(popup->grid), 
+                                            te->grid_left,
+                                            te->grid_top - 1);
+      display_widget (popup, w);
+    }
+}
+
 MetaTabEntryKey
 meta_ui_tab_popup_get_selected (MetaTabPopup *popup)
 {
@@ -593,6 +688,46 @@ meta_ui_tab_popup_select (MetaTabPopup *popup,
     }
 }
 
+GtkWidget*
+meta_ui_tab_popup_get_widget (MetaTabPopup *popup)
+{
+  if (popup != NULL)
+    return popup->window;
+  else
+    return NULL;
+}
+
+void
+meta_ui_tab_popup_mouse_press (MetaTabPopup       *popup,
+                               gint                x,
+                               gint                y)
+{
+  GList *tmp = popup->entries;
+  gboolean found = FALSE;
+  while (tmp != NULL && !found)
+    {
+      TabEntry *te = tmp->data;
+      gint wx, wy;
+      if (gtk_widget_translate_coordinates(popup->window,
+                                           te->widget,
+                                           x, y,
+                                           &wx, &wy))
+        {
+          GtkAllocation alloc;
+          gtk_widget_get_allocation(te->widget, &alloc);
+          found = (0 <= wx && wx < alloc.width &&
+                   0 <= wy && wy < alloc.height);
+          if (found)
+            {
+              popup->current = tmp;
+              display_entry (popup, te);
+            }
+        }
+      tmp = tmp->next;
+    }
+}
+
+
 #define META_TYPE_SELECT_IMAGE            (meta_select_image_get_type ())
 #define META_SELECT_IMAGE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), META_TYPE_SELECT_IMAGE, MetaSelectImage))
 
@@ -614,14 +749,28 @@ struct _MetaSelectImageClass
 static GType meta_select_image_get_type (void) G_GNUC_CONST;
 
 static GtkWidget*
-selectable_image_new (GdkPixbuf *pixbuf)
+selectable_image_new (GdkPixbuf *pixbuf, cairo_surface_t *win_surface)
 {
-  GtkWidget *w;
+  GtkWidget *widget;
 
-  w = g_object_new (meta_select_image_get_type (), NULL);
-  gtk_image_set_from_pixbuf (GTK_IMAGE (w), pixbuf);
+  widget = g_object_new (meta_select_image_get_type (), NULL);
 
-  return w;
+  if (win_surface == NULL)
+    {
+      int scale;
+      cairo_surface_t *surface;
+
+      scale = gtk_widget_get_scale_factor (widget);
+      surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale, NULL);
+
+      gtk_image_set_from_surface (GTK_IMAGE (widget), surface);
+
+      cairo_surface_destroy (surface);
+    }
+  else
+    gtk_image_set_from_surface (GTK_IMAGE (widget), win_surface);
+
+  return widget;
 }
 
 static void
@@ -638,7 +787,9 @@ unselect_image (GtkWidget *widget)
   gtk_widget_queue_draw (widget);
 }
 
-static void     meta_select_image_class_init   (MetaSelectImageClass *klass);
+static void     meta_select_image_class_init   (MetaSelectImageClass *klass,
+                                                void                 *data);
+
 static gboolean meta_select_image_draw         (GtkWidget            *widget,
                                                 cairo_t              *cr);
 
@@ -671,7 +822,33 @@ meta_select_image_get_type (void)
 }
 
 static void
-meta_select_image_class_init (MetaSelectImageClass *klass)
+meta_select_image_get_preferred_width (GtkWidget *widget,
+                                       gint      *minimum_width,
+                                       gint      *natural_width)
+{
+  GTK_WIDGET_CLASS (parent_class)->get_preferred_width (widget,
+                                                        minimum_width,
+                                                        natural_width);
+
+  *minimum_width += (INSIDE_SELECT_RECT + OUTSIDE_SELECT_RECT) * 2;
+  *natural_width += (INSIDE_SELECT_RECT + OUTSIDE_SELECT_RECT) * 2;
+}
+
+static void
+meta_select_image_get_preferred_height (GtkWidget *widget,
+                                        gint      *minimum_height,
+                                        gint      *natural_height)
+{
+  GTK_WIDGET_CLASS (parent_class)->get_preferred_height (widget,
+                                                         minimum_height,
+                                                         natural_height);
+
+  *minimum_height += (INSIDE_SELECT_RECT + OUTSIDE_SELECT_RECT) * 2;
+  *natural_height += (INSIDE_SELECT_RECT + OUTSIDE_SELECT_RECT) * 2;
+}
+
+static void
+meta_select_image_class_init (MetaSelectImageClass *klass, void *data)
 {
   GtkWidgetClass *widget_class;
 
@@ -680,56 +857,47 @@ meta_select_image_class_init (MetaSelectImageClass *klass)
   widget_class = GTK_WIDGET_CLASS (klass);
 
   widget_class->draw = meta_select_image_draw;
+  widget_class->get_preferred_width = meta_select_image_get_preferred_width;
+  widget_class->get_preferred_height = meta_select_image_get_preferred_height;
 }
 
 static gboolean
 meta_select_image_draw (GtkWidget *widget,
                         cairo_t   *cr)
 {
-  GtkAllocation allocation;
+  GtkStyleContext *context;
 
-  gtk_widget_get_allocation (widget, &allocation);
+  context = gtk_widget_get_style_context (widget);
 
   if (META_SELECT_IMAGE (widget)->selected)
     {
-      GtkMisc *misc;
       GtkRequisition requisition;
-      GtkStyleContext *context;
       GdkRGBA color;
       int x, y, w, h;
-      gint xpad, ypad;
-      gfloat xalign, yalign;
 
-      misc = GTK_MISC (widget);
+      gtk_widget_get_preferred_size (widget, &requisition, 0);
 
-      gtk_widget_get_preferred_size (widget, &requisition, NULL);
-      gtk_misc_get_alignment (misc, &xalign, &yalign);
-      gtk_misc_get_padding (misc, &xpad, &ypad);
+      x = INSIDE_SELECT_RECT;
+      y = INSIDE_SELECT_RECT;
 
-      x = (allocation.width - (requisition.width - xpad * 2)) * xalign + 0.5;
-      y = (allocation.height - (requisition.height - ypad * 2)) * yalign + 0.5;
+      w = requisition.width - OUTSIDE_SELECT_RECT * 2;
+      h = requisition.height - OUTSIDE_SELECT_RECT * 2;
 
-      x -= INSIDE_SELECT_RECT + 1;
-      y -= INSIDE_SELECT_RECT + 1;
+      gtk_style_context_set_state (context, GTK_STATE_FLAG_SELECTED);
+      meta_gtk_style_get_light_color (context, GTK_STATE_FLAG_SELECTED, &color);
 
-      w = requisition.width - OUTSIDE_SELECT_RECT * 2 - 1;
-      h = requisition.height - OUTSIDE_SELECT_RECT * 2 - 1;
-
-      context = gtk_widget_get_style_context (widget);
-
-      gtk_style_context_set_state (context,
-                                   gtk_widget_get_state_flags (widget));
-
-      gtk_style_context_lookup_color (context, "color", &color);
-
-      cairo_set_line_width (cr, 2.0);
+      /* We set the line width absurdly high to overflow it behind the icon. */
+      cairo_set_line_width (cr, 256.0);
       cairo_set_source_rgb (cr, color.red, color.green, color.blue);
 
-      cairo_rectangle (cr, x, y, w + 1, h + 1);
+      cairo_rectangle (cr, x, y, w, h);
       cairo_stroke (cr);
-
       cairo_set_line_width (cr, 1.0);
     }
+  else
+    {
+      gtk_style_context_set_state (context, GTK_STATE_FLAG_NORMAL);
+    }
 
   return GTK_WIDGET_CLASS (parent_class)->draw (widget, cr);
 }
@@ -756,23 +924,37 @@ struct _MetaSelectWorkspaceClass
 static GType meta_select_workspace_get_type (void) G_GNUC_CONST;
 
 #define SELECT_OUTLINE_WIDTH 2
-#define MINI_WORKSPACE_WIDTH 48
+#define MINI_WORKSPACE_SCREEN_FRACTION 0.33
 
 static GtkWidget*
-selectable_workspace_new (MetaWorkspace *workspace)
+selectable_workspace_new (MetaWorkspace *workspace, int entry_count)
 {
   GtkWidget *widget;
-  double screen_aspect;
+  const MetaXineramaScreenInfo *current;
+  int mini_workspace_width, mini_workspace_height;
+  double mini_workspace_ratio;
 
   widget = g_object_new (meta_select_workspace_get_type (), NULL);
 
-  screen_aspect = (double) workspace->screen->rect.height /
-                  (double) workspace->screen->rect.width;
+  current = meta_screen_get_current_xinerama (workspace->screen);
+
+  if (workspace->screen->rect.width < workspace->screen->rect.height)
+  {
+    mini_workspace_ratio = (double) workspace->screen->rect.width / (double) workspace->screen->rect.height;
+    mini_workspace_height = (int) ((double) current->rect.height * MINI_WORKSPACE_SCREEN_FRACTION / entry_count);
+    mini_workspace_width = (int) ((double) mini_workspace_height * mini_workspace_ratio);
+  }
+  else
+  {
+    mini_workspace_ratio = (double) workspace->screen->rect.height / (double) workspace->screen->rect.width;
+    mini_workspace_width = (int) ((double) current->rect.width * MINI_WORKSPACE_SCREEN_FRACTION / entry_count);
+    mini_workspace_height = (int) ((double) mini_workspace_width * mini_workspace_ratio);
+  }
 
   /* account for select rect */
   gtk_widget_set_size_request (widget,
-                               MINI_WORKSPACE_WIDTH + SELECT_OUTLINE_WIDTH * 2,
-                               MINI_WORKSPACE_WIDTH * screen_aspect + SELECT_OUTLINE_WIDTH * 2);
+                               mini_workspace_width + SELECT_OUTLINE_WIDTH * 2,
+                               mini_workspace_height + SELECT_OUTLINE_WIDTH * 2);
 
   META_SELECT_WORKSPACE (widget)->workspace = workspace;
 
@@ -793,7 +975,8 @@ unselect_workspace (GtkWidget *widget)
   gtk_widget_queue_draw (widget);
 }
 
-static void meta_select_workspace_class_init (MetaSelectWorkspaceClass *klass);
+static void meta_select_workspace_class_init (MetaSelectWorkspaceClass *klass,
+                                              void                     *data);
 
 static gboolean meta_select_workspace_draw (GtkWidget *widget,
                                             cairo_t   *cr);
@@ -828,7 +1011,8 @@ meta_select_workspace_get_type (void)
 }
 
 static void
-meta_select_workspace_class_init (MetaSelectWorkspaceClass *klass)
+meta_select_workspace_class_init (MetaSelectWorkspaceClass *klass,
+                                  void                     *data)
 {
   GtkWidgetClass *widget_class;
 
@@ -942,7 +1126,7 @@ meta_select_workspace_draw (GtkWidget *widget,
       gtk_style_context_set_state (context,
                                    gtk_widget_get_state_flags (widget));
 
-      gtk_style_context_lookup_color (context, "color", &color);
+      meta_gtk_style_get_light_color (context, GTK_STATE_FLAG_SELECTED, &color);
 
       cairo_set_line_width (cr, SELECT_OUTLINE_WIDTH);
       cairo_set_source_rgb (cr, color.red, color.green, color.blue);
